/* Copyright (c) Mark J. Kilgard, 1997. */

/* This program is freely distributable without licensing fees  and is
   provided without guarantee or warrantee expressed or  implied. This
   program is -not- in the public domain. */

#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include "GLES2/gl2.h"
#include "GLES2/gl2ext.h"

#include "priv.h"
#include "nvtexfont2.h"

#ifdef USE_BINARY_SHADERS

static GLubyte rastVertBin[] = {
#include "vtxraster.cghex"
};

static GLubyte rastFragBin[] = {
#include "colraster.cghex"
};

static GLubyte vecVertBin[] = {
#include "vtxvector.cghex"
};

static GLubyte vecFragBin[] = {
#include "colvector.cghex"
};

#else

static GLubyte rastVert[] = {
#include "vtxraster.hglslv"
};

static GLubyte rastFrag[] = {
#include "colraster.hglslf"
};

static GLubyte vecVert[] = {
#include "vtxvector.hglslv"
};

static GLubyte vecFrag[] = {
#include "colvector.hglslf"
};

#endif

GLuint rasterProg = 0, vectorProg = 0;
char *lastError;

/* General Calls */

#ifndef USE_BINARY_SHADERS
GLuint loadShaderSourcesString(
    char* vertSource, GLuint vertSourceLen,
    char* fragSource, GLuint fragSourceLen)
{
    GLuint prog = 0;
    GLuint vertShader;
    GLuint fragShader;

    // Load the shader files
    if (!vertSource || !fragSource) goto done;

    // Create the program
    prog = glCreateProgram();

    // Create the GL shader objects
    vertShader = glCreateShader(GL_VERTEX_SHADER);
    fragShader = glCreateShader(GL_FRAGMENT_SHADER);

    // Load shader sources into GL and compile
    glShaderSource(vertShader, 1, (const char**)&vertSource, &vertSourceLen);
    glCompileShader(vertShader);
    glShaderSource(fragShader, 1, (const char**)&fragSource, &fragSourceLen);
    glCompileShader(fragShader);

    // Attach the shaders to the program
    glAttachShader(prog, vertShader);
    glAttachShader(prog, fragShader);

    // Delete the shaders
    glDeleteShader(vertShader);
    glDeleteShader(fragShader);

    // Link and validate the shader program
    glLinkProgram(prog);
    glValidateProgram(prog);

    done:

    return prog;
}
#endif

void deinitShaders(void)
{
    if (vectorProg) {
        glDeleteProgram(vectorProg);
        vectorProg = 0;

    }

    if (rasterProg) {
        glDeleteProgram(rasterProg);
        rasterProg = 0;
    }
}

void initShaders(void)
{

#   ifdef USE_BINARY_SHADERS

    GLuint rastVertShad, rastFragShad;
    GLuint vecVertShad, vecFragShad;

    // set up Raster shaders
    rastVertShad = glCreateShader(GL_VERTEX_SHADER);
    glShaderBinary(1, &rastVertShad, GL_NVIDIA_PLATFORM_BINARY_NV, rastVertBin, sizeof rastVertBin);

    rastFragShad = glCreateShader(GL_FRAGMENT_SHADER);
    glShaderBinary(1, &rastFragShad, GL_NVIDIA_PLATFORM_BINARY_NV, rastFragBin, sizeof rastFragBin);
    rasterProg = glCreateProgram();

    glAttachShader(rasterProg, rastVertShad);
    glAttachShader(rasterProg, rastFragShad);

    // Shader objects deletion is deferred until program object deletion
    glDeleteShader(rastVertShad);
    glDeleteShader(rastFragShad);

    glBindAttribLocation(rasterProg, VERT_ARRAY, "vertArray");
    glBindAttribLocation(rasterProg, TEX_ARRAY, "texArray");

    glLinkProgram(rasterProg);

    // set up Vector shaders
    vecVertShad = glCreateShader(GL_VERTEX_SHADER);
    glShaderBinary(1, &vecVertShad, GL_NVIDIA_PLATFORM_BINARY_NV, vecVertBin, sizeof vecVertBin);

    vecFragShad = glCreateShader(GL_FRAGMENT_SHADER);
    glShaderBinary(1, &vecFragShad, GL_NVIDIA_PLATFORM_BINARY_NV, vecFragBin, sizeof vecFragBin);
    vectorProg = glCreateProgram();

    glAttachShader(vectorProg, vecVertShad);
    glAttachShader(vectorProg, vecFragShad);

    glDeleteShader(vecVertShad);
    glDeleteShader(vecFragShad);

    glBindAttribLocation(vectorProg, VERT_ARRAY, "vertArray");

    glLinkProgram(vectorProg);

#   else

    rasterProg = loadShaderSourcesString(rastVert, sizeof(rastVert), rastFrag, sizeof(rastFrag));
    vectorProg = loadShaderSourcesString(vecVert, sizeof(vecVert), vecFrag, sizeof(vecFrag));

    glBindAttribLocation(rasterProg, VERT_ARRAY, "vertArray");
    glBindAttribLocation(rasterProg, TEX_ARRAY, "texArray");
    glBindAttribLocation(vectorProg, VERT_ARRAY, "vertArray");

#   endif
}

char *nvtexfont2ErrorString(void)
{
    return lastError;
}

NVTexfont2Context nvtexfont2AllocContext(void)
{
    nvtexfont2State *state = malloc(sizeof(nvtexfont2State));

    state->r = 1.0; state->g = 0.0; state->b = 0.0;
    state->scale = 1.0;
    state->x = 0.0; state->y = 0.0;

    return (NVTexfont2Context)state;
}

void nvtexfont2FreeContext(NVTexfont2Context tfc)
{
    free((nvtexfont2State *)tfc);
}

void nvtexfont2SetContextPos(NVTexfont2Context tfc, float x, float y)
{
    ((nvtexfont2State *)tfc)->x = x;
    ((nvtexfont2State *)tfc)->y = y;
}

void nvtexfont2SetContextScale(NVTexfont2Context tfc, float scale)
{
    ((nvtexfont2State *)tfc)->scale = scale;
}

void nvtexfont2SetContextColor(NVTexfont2Context tfc, float r, float g, float b)
{
    ((nvtexfont2State *)tfc)->r = r;
    ((nvtexfont2State *)tfc)->g = g;
    ((nvtexfont2State *)tfc)->b = b;
}
